package ru.dbotthepony.mc.otm.block;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.world.item.context.BlockPlaceContext;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.EntityBlock;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.entity.BlockEntityTicker;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.phys.shapes.BooleanOp;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.world.phys.shapes.Shapes;
import net.minecraft.world.phys.shapes.VoxelShape;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.block.entity.BlockEntityBatteryBank;
import ru.dbotthepony.mc.otm.shapes.BlockShapes;

import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import java.util.List;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class BlockBatteryBank extends BlockMatteryRotatable implements EntityBlock {
	public static final BooleanProperty[] BATTERY_SLOTS_PROPS = new BooleanProperty[] {
		BooleanProperty.create("battery_0"),
		BooleanProperty.create("battery_1"),
		BooleanProperty.create("battery_2"),
		BooleanProperty.create("battery_3"),
		BooleanProperty.create("battery_4"),
		BooleanProperty.create("battery_5"),
		BooleanProperty.create("battery_6"),
		BooleanProperty.create("battery_7"),
		BooleanProperty.create("battery_8"),
		BooleanProperty.create("battery_9"),
		BooleanProperty.create("battery_10"),
		BooleanProperty.create("battery_11"),
	};

	@Nullable
	@Override
	public BlockState getStateForPlacement(BlockPlaceContext context) {
		var state = super.getStateForPlacement(context);

		for (var prop : BATTERY_SLOTS_PROPS)
			state = state.setValue(prop, false);

		return state;
	}

	@Nullable
	@Override
	public <T extends BlockEntity> BlockEntityTicker<T> getTicker(Level level, BlockState p_153213_, BlockEntityType<T> type) {
		return level.isClientSide || type != Registry.BlockEntities.BATTERY_BANK ? null : BlockEntityBatteryBank::tick;
	}

	@Override
	protected void createBlockStateDefinition(StateDefinition.Builder<Block, BlockState> builder) {
		builder.add(BATTERY_SLOTS_PROPS);
		super.createBlockStateDefinition(builder);
	}

	@Nullable
	@Override
	public BlockEntity newBlockEntity(BlockPos blockPos, BlockState blockState) {
		return new BlockEntityBatteryBank(blockPos, blockState);
	}

	private static final List<VoxelShape> SHAPES;

	static {
		var def = BlockShapes.BATTERY_BANK.computeShape();

		SHAPES = List.of(
			def,
			def,
			def,
			BlockShapes.BATTERY_BANK.rotate(Direction.NORTH).computeShape(),
			BlockShapes.BATTERY_BANK.rotate(Direction.WEST).computeShape(),
			BlockShapes.BATTERY_BANK.rotate(Direction.EAST).computeShape()
		);
	}

	@Override
	@SuppressWarnings("deprecation")
	public VoxelShape getShape(BlockState p_60555_, BlockGetter p_60556_, BlockPos p_60557_, CollisionContext p_60558_) {
		return SHAPES.get(p_60555_.getValue(FACING).ordinal());
	}

	@Override
	public boolean faceToPlayer(BlockPlaceContext context) {
		return false;
	}
}
