package ru.dbotthepony.mc.otm.block.entity;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.energy.CapabilityEnergy;
import net.minecraftforge.energy.IEnergyStorage;
import ru.dbotthepony.mc.otm.capability.IMatteryEnergyStorage;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.MatteryMachineEnergyStorage;
import ru.dbotthepony.mc.otm.container.MatteryContainer;
import ru.dbotthepony.mc.otm.core.Fraction;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import java.math.BigDecimal;
import java.util.Optional;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
abstract public class BlockEntityMatteryPowered extends BlockEntityMattery {
	protected MatteryMachineEnergyStorage energy = null;
	private LazyOptional<MatteryMachineEnergyStorage> energy_resolver = LazyOptional.of(() -> energy);
	private boolean valid = true;

	public MatteryContainer battery_container = new MatteryContainer(this::setChanged, 1);

	protected void batteryChargeLoop() {
		if (energy == null)
			return;

		if (energy.getMissingPower().compareTo(Fraction.ZERO) <= 0)
			return;

		var demand = energy.getMissingPower();

		if (demand.compareTo(Fraction.ZERO) == 0)
			return;

		demand = demand.min(energy.receiveEnergyOuter(demand, true));

		for (int i = 0; i < battery_container.getContainerSize() && demand.compareTo(Fraction.ZERO) > 0; i++) {
			final var stack = battery_container.getItem(i);

			if (!stack.isEmpty()) {
				final var mattery_storage = stack.getCapability(MatteryCapability.ENERGY).resolve();

				if (mattery_storage.isPresent()) {
					final var drain = mattery_storage.get().extractEnergyOuter(demand, true);

					if (drain.compareTo(Fraction.ZERO) != 0) {
						final var receive = energy.receiveEnergyOuter(drain, true);

						mattery_storage.get().extractEnergyOuter(receive, false);
						energy.receiveEnergyOuter(receive, false);

						demand = demand.minus(receive);
					}
				} else {
					final var storage = stack.getCapability(CapabilityEnergy.ENERGY).resolve();

					if (storage.isPresent()) {
						final var drain = MatteryCapability.drainFE(storage.get(), demand, true);

						if (drain.compareTo(Fraction.ZERO) != 0) {
							final var receive = energy.receiveEnergyOuter(drain, true);

							MatteryCapability.drainFE(storage.get(), receive, false);
							energy.receiveEnergyOuter(receive, false);

							demand = demand.minus(receive);
						}
					}
				}
			}
		}
	}

	public BlockEntityMatteryPowered(BlockEntityType<?> p_155228_, BlockPos p_155229_, BlockState p_155230_) {
		super(p_155228_, p_155229_, p_155230_);
	}

	@Override
	public void invalidateCaps() {
		super.invalidateCaps();
		valid = false;
		energy_resolver.invalidate();
	}

	@Override
	public void reviveCaps() {
		super.reviveCaps();
		valid = true;
		energy_resolver = LazyOptional.of(() -> energy);
	}

	@Nonnull
	public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
		if (valid && (cap == MatteryCapability.ENERGY || cap == CapabilityEnergy.ENERGY) && energy != null) {
			return energy_resolver.cast();
		}

		return super.getCapability(cap, side);
	}

	@Override
	public void saveAdditional(CompoundTag nbt) {
		super.saveAdditional(nbt);

		if (energy != null)
			nbt.put("energy_cap", energy.serializeNBT());

		nbt.put("battery_container", battery_container.serializeNBT());
	}

	public void load(CompoundTag nbt) {
		if (energy != null && nbt.get("energy_cap") instanceof CompoundTag tag)
			energy.deserializeNBT(tag);

		if (nbt.contains("battery_container") && nbt.get("battery_container") instanceof CompoundTag tag)
			battery_container = MatteryContainer.of(this::setChanged, tag, 1);

		super.load(nbt);
	}
}
