package ru.dbotthepony.mc.otm.capability.drive;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraftforge.registries.RegistryManager;
import ru.dbotthepony.mc.otm.core.Fraction;
import ru.dbotthepony.mc.otm.storage.IStorageTuple;
import ru.dbotthepony.mc.otm.storage.ItemStackWrapper;
import ru.dbotthepony.mc.otm.storage.StorageObjectRegistry;
import ru.dbotthepony.mc.otm.storage.StorageObjectTuple;

import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class ItemMatteryDrive extends AbstractMatteryDrive<ItemStackWrapper> implements IItemMatteryDrive {
	private static StorageObjectTuple<ItemStackWrapper> identity;

	public ItemMatteryDrive(Fraction capacity, int max_different_stacks) {
		super(capacity, max_different_stacks);
	}

	public ItemMatteryDrive(Fraction capacity) {
		super(capacity);
	}

	@Override
	public StorageObjectTuple<ItemStackWrapper> identity() {
		if (identity == null)
			identity = StorageObjectRegistry.getOrError(ItemStackWrapper.class);

		return identity;
	}

	public ItemStack insertObject(ItemStack item, boolean simulate) {
		return insertStack(new ItemStackWrapper(item), simulate).stack();
	}

	@Override
	protected CompoundTag serializeStack(IStorageTuple<ItemStackWrapper> item) {
		final var tag = new CompoundTag();
		final var location = Objects.requireNonNull(item.stack().stack().getItem().getRegistryName(), "Missing registry name for stored Item").toString();

		tag.putString("item", location);
		tag.putInt("count", item.stack().stack().getCount());

		CompoundTag item_tag;

		if ((item_tag = item.stack().stack().getTag()) != null) {
			tag.put("data", item_tag);
		}

		return tag;
	}

	@Nullable
	@Override
	protected ItemStackWrapper deserializeStack(CompoundTag tag) {
		final var item = RegistryManager.ACTIVE.getRegistry(Item.class).getValue(new ResourceLocation(tag.getString("item")));

		if (item != null && item != Items.AIR) {
			final var count = tag.getInt("count");
			final var data = tag.get("data");

			final var itemstack = new ItemStack(item, count);

			if (data != null) {
				itemstack.setTag((CompoundTag) data);
			}

			return new ItemStackWrapper(itemstack);
		}

		return null;
	}


	@Override
	public List<IStorageTuple<ItemStackWrapper>> findItems(Item item) {
		var list = items.get(item);

		if (list != null)
			return List.copyOf(list);

		return List.of();
	}

	@Override
	public List<IStorageTuple<ItemStackWrapper>> findItems(ItemStack stack) {
		var list = items.get(stack.getItem());

		if (list == null)
			return List.of();

		var amount = 0;

		for (var _stack : list) {
			if (ItemStack.tagMatches(_stack.stack().stack(), stack)) {
				amount++;
			}
		}

		var build_list = new ArrayList<IStorageTuple<ItemStackWrapper>>(amount);
		var i = 0;

		for (var _stack : list) {
			if (ItemStack.tagMatches(_stack.stack().stack(), stack)) {
				build_list.set(i, _stack);
				i++;
			}
		}

		return build_list;
	}
}
