package ru.dbotthepony.mc.otm.client.screen.panels;

import com.mojang.blaze3d.vertex.PoseStack;
import ru.dbotthepony.mc.otm.client.screen.MatteryScreen;
import ru.dbotthepony.mc.otm.client.render.RenderHelper;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.function.Consumer;
import java.util.function.Supplier;

public class ScrollBarPanel extends EditablePanel {
	public class ScrollBarButtonPanel extends EditablePanel {
		public ScrollBarButtonPanel() {
			super(ScrollBarPanel.this.screen, ScrollBarPanel.this, 1, 1, 12, 15);
		}

		private boolean is_scrolling;

		public boolean isScrolling() {
			return is_scrolling;
		}

		@Override
		protected void innerRender(PoseStack stack, float mouse_x, float mouse_y, float flag) {
			if (is_scrolling) {
				RenderHelper.scroll_bar_button_press.render(stack, 0, 0);
			} else if (isHovered()) {
				RenderHelper.scroll_bar_button_hover.render(stack, 0, 0);
			} else {
				RenderHelper.scroll_bar_button.render(stack, 0, 0);
			}
		}

		private double last_mouse_y;

		@Override
		protected boolean mouseClickedInner(double mouse_x, double mouse_y, int flag) {
			is_scrolling = true;
			setIgnoreMouseEventBoundaries(true);
			last_mouse_y = mouse_y;
			return true;
		}

		@Override
		protected boolean mouseReleasedInner(double mouse_x, double mouse_y, int flag) {
			if (is_scrolling) {
				is_scrolling = false;
				setIgnoreMouseEventBoundaries(false);
				scroll = getScroll();
				return true;
			}

			return false;
		}

		@Override
		protected boolean mouseScrolledInner(double mouse_x, double mouse_y, double scroll) {
			return ScrollBarPanel.this.mouseScrolledInner(mouse_x, mouse_y, scroll);
		}

		@Override
		protected boolean mouseDraggedInner(double mouse_x, double mouse_y, int flag, double drag_x, double drag_y) {
			if (is_scrolling) {
				var diff = mouse_y - last_mouse_y;
				last_mouse_y = mouse_y;
				setScrollInternal((float) (scroll + diff / ScrollBarPanel.this.getHeight()));
				updateScrollBarPosition();
				return true;
			}

			return false;
		}
	}

	public static final int WIDTH = 14;

	public final ScrollBarButtonPanel scroll_button = new ScrollBarButtonPanel();
	public Consumer<Float> scroll_callback;

	@Nonnull
	public Supplier<Float> scroll_multiplier = () -> 0.1f;

	public ScrollBarPanel(@Nonnull MatteryScreen<?> screen, @Nullable EditablePanel parent, float x, float y, float height) {
		super(screen, parent, x, y, WIDTH, height);
	}

	public void setupRowMultiplier(Supplier<Integer> row_supplier) {
		scroll_multiplier = () -> {
			int rows = row_supplier.get();

			if (rows < 1) {
				return 0f;
			}

			return 1f / rows;
		};
	}

	protected float scroll;

	protected void onScrolled(float new_scroll, float old_scroll) {
		if (scroll_callback != null)
			scroll_callback.accept(new_scroll);
	}

	public float getScroll() {
		return scroll < 0 ? 0 : scroll > 1 ? 1 : scroll;
	}

	public void setScroll(float scroll) {
		var previous = this.scroll;

		this.scroll = scroll < 0 ? 0 : scroll > 1 ? 1 : scroll;

		if (previous != this.scroll) {
			onScrolled(this.scroll, previous);
		}
	}

	protected void setScrollInternal(float scroll) {
		var previous = getScroll();
		this.scroll = scroll;
		var current = getScroll();

		if (previous != current) {
			onScrolled(current, previous);
		}
	}

	public void updateScrollBarPosition() {
		scroll_button.setY(1 + (getHeight() - 2 - scroll_button.getHeight()) * getScroll());
	}

	@Override
	protected void performLayout() {
		super.performLayout();

		updateScrollBarPosition();
	}

	public void translateScrollValue(int new_max, int old_max) {
		if (new_max < 1 || old_max < 1) {
			return;
		}

		scroll = (scroll * old_max) / new_max;
	}

	public int getScroll(int max_rows) {
		if (max_rows <= 1) {
			return 0;
		}

		return (int) Math.floor(getScroll() * max_rows + 0.5);
	}

	// public is for emulating input from outside
	@Override
	public boolean mouseScrolledInner(double mouse_x, double mouse_y, double scroll) {
		setScrollInternal((float) (this.scroll - scroll * scroll_multiplier.get()));
		updateScrollBarPosition();
		return true;
	}

	@Override
	protected void innerRender(PoseStack stack, float mouse_x, float mouse_y, float flag) {
		RenderHelper.drawScrollBarBackground(stack, 0, 0, getHeight());


	}
}
