package ru.dbotthepony.mc.otm.menu.data;

import net.minecraft.client.Minecraft;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.network.NetworkEvent;

import java.util.function.Supplier;

public record StackAddPacket(int id, int stack_id, ItemStack stack) {
	public void write(FriendlyByteBuf buffer) {
		buffer.writeInt(id);
		buffer.writeInt(stack_id);
		buffer.writeRegistryId(stack.getItem());
		buffer.writeInt(stack.getCount());
		buffer.writeBoolean(stack.getTag() != null);

		if (stack.getTag() != null)
			buffer.writeNbt(stack.getShareTag());
	}

	public static StackAddPacket read(FriendlyByteBuf buffer) {
		var id = buffer.readInt();
		var stack = buffer.readInt();
		var item = buffer.readRegistryIdSafe(Item.class);
		var count = buffer.readInt();

		var state = new ItemStack(item, count);

		if (buffer.readBoolean())
			state.readShareTag(buffer.readNbt());

		return new StackAddPacket(id, stack, state);
	}

	public void play(Supplier<NetworkEvent.Context> context) {
		context.get().setPacketHandled(true);
		context.get().enqueueWork(() -> {
			var get = Minecraft.getInstance().player.containerMenu instanceof INetworkedItemViewSupplier supplier && Minecraft.getInstance().player.containerMenu.containerId == id ? supplier.getNetworkedItemView() : null;

			if (get == null) {
				throw new IllegalStateException("No such item tracker with id " + id);
			}

			if (get.state.containsKey(stack_id)) {
				throw new IllegalStateException("Item tracker " + id + " already has stack with id of " + stack_id);
			}

			get.state.put(stack_id, new NetworkedItemView.NetworkedItem(stack_id, stack));
			get.clearCache();
		});
	}
}
