package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.MethodsReturnNonnullByDefault
import javax.annotation.ParametersAreNonnullByDefault
import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.world.level.block.state.BlockState
import ru.dbotthepony.mc.otm.block.entity.worker.BlockEntityMatteryWorker
import ru.dbotthepony.mc.otm.capability.matter.MatterHandlerCapability
import ru.dbotthepony.mc.otm.capability.matter.IMatterHandler
import ru.dbotthepony.mc.otm.container.MatteryContainer
import net.minecraft.world.item.ItemStack
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import ru.dbotthepony.mc.otm.menu.MatterReplicatorMenu
import ru.dbotthepony.mc.otm.block.entity.worker.MachineJob
import ru.dbotthepony.mc.otm.block.entity.worker.MachineJobStatus
import ru.dbotthepony.mc.otm.capability.matter.MatterTask
import ru.dbotthepony.mc.otm.capability.matter.PatternState
import ru.dbotthepony.mc.otm.matter.MatterRegistry
import ru.dbotthepony.mc.otm.block.entity.worker.WorkTickContext
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import net.minecraftforge.items.CapabilityItemHandler
import ru.dbotthepony.mc.otm.capability.MatteryMachineEnergyStorage
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.level.Level
import net.minecraftforge.common.capabilities.Capability
import ru.dbotthepony.mc.otm.Registry
import ru.dbotthepony.mc.otm.core.Fraction
import ru.dbotthepony.mc.otm.graph.Graph6Node
import ru.dbotthepony.mc.otm.graph.matter.IMatterGraphNode
import ru.dbotthepony.mc.otm.graph.matter.MatterNetworkGraph
import ru.dbotthepony.mc.otm.ifHas
import ru.dbotthepony.mc.otm.set

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
class BlockEntityMatterReplicator(p_155229_: BlockPos, p_155230_: BlockState) :
	BlockEntityMatteryWorker(Registry.BlockEntities.MATTER_REPLICATOR, p_155229_, p_155230_), IMatterGraphNode {

	private val node = Graph6Node<IMatterGraphNode>(this)
	private val resolverNode = LazyOptional.of { this }

	override fun getAsMatterNode(): Graph6Node<IMatterGraphNode> {
		return node
	}

	@JvmField
	val matter = MatterHandlerCapability(this::setChangedLight, IMatterHandler.MatterDirection.RECEIVE, Fraction(2))

	// обычные запросы
	@JvmField
	val regularSlots = MatteryContainer(this::setChangedLight, 3)

	// запросы от matter replicator interface?
	val reservedSlots = MatteryContainer(this::setChangedLight, 3)

	private val itemHandler = regularSlots.handler { slot: Int, stack: ItemStack? -> false }

	override fun getDefaultDisplayName(): Component {
		return NAME
	}

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return MatterReplicatorMenu(containerID, inventory, this)
	}

	override fun getBaseConsumption(): Fraction {
		return BASE_CONSUMPTION
	}

	override fun onJobFinish(job: MachineJob): MachineJobStatus {
		if (!regularSlots.fullyAddItem(job.stack())) {
			return MachineJobStatus(false, 20)
		}

		(node.graph as MatterNetworkGraph?)?.notifyTaskCompletion(MatterTask.deserializeNBT(job.data["task"])!!)
		return MachineJobStatus()
	}

	override fun onMatterTaskCreated(task: MatterTask) {
		is_idling = false
	}

	override fun onMatterTaskUpdated(new_state: MatterTask, old_state: MatterTask) {
		is_idling = false
	}

	override fun onPatternAdded(state: PatternState) {
		is_idling = false
	}

	override fun setRemoved() {
		super.setRemoved()
		node.destroy(::MatterNetworkGraph)
	}

	override fun setLevel(p_155231_: Level) {
		super.setLevel(p_155231_)

		if (p_155231_ is ServerLevel)
			MatterNetworkGraph.discoverFull(this, node)
	}

	override fun getNextJob(): MachineJob? {
		val graph = node.graph as MatterNetworkGraph? ?: return null
		val allocation = graph.allocateTask(false) ?: return null
		val stack = allocation.task().stack(1)

		// ????????
		if (!MatterRegistry.hasMatterValue(stack)) return null
		val job = MachineJob(stack, MatterRegistry.getMatterValue(stack).toDouble() * TICKS_PER_MTU)

		job.data["task"] = allocation.task.serializeNBT()
		if (allocation.pattern != null) job.data["pattern"] = allocation.pattern!!.serializeNBT()

		return job
	}

	override fun onWorkTick(context: WorkTickContext): MachineJobStatus {
		val drainPerTick = MTU_PER_TICK_BD.times(context.work_speed())
		val graph = node.graph as MatterNetworkGraph? ?: return MachineJobStatus(false, 20)

		if (matter.extractMatterInner(drainPerTick, true) < drainPerTick) {
			// в машине недостаточно материи

			if (drainPerTick > matter.maxStoredMatter) {
				// в тик требуется больше материи, чем её может хранить репликатор
				val toExtract = drainPerTick - matter.extractMatterInner(drainPerTick, true)
				val drain = graph.extractMatter(toExtract, true)

				if (drain < toExtract) {
					// недостаточно материи в сети
					return MachineJobStatus(false, 200)
				}

				// достаточно материи в сети + внутри машины
				matter.extractMatterInner(drainPerTick, false)
				graph.extractMatter(drain, false)
				return MachineJobStatus()
			} else {
				// в тик требуется меньше материи, чем её может хранить репликатор
				// примем из сети недостающее количество бака материи, или 200 тиков репликации, что меньше
				val toExtract =
					matter.missingMatter.min(drainPerTick.times(DRAIN_MULT))

				val drain = graph.extractMatter(toExtract, true)

				if (drain.isZero()) {
					// в сети нет материи
					return MachineJobStatus(false, 200)
				}

				val received = matter.receiveMatterOuter(drain, false)
				graph.extractMatter(received, false)

				// получили материю, проверяем возможность работы
				if (matter.extractMatterInner(drainPerTick, false) >= drainPerTick) {
					return MachineJobStatus()
				} else {
					// :(
					return MachineJobStatus(false, 200)
				}
			}
		}

		// в машине достаточно материи
		matter.extractMatterInner(drainPerTick, false)
		return MachineJobStatus()
	}

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt["regular_slots"] = regularSlots.serializeNBT()
		nbt["reserved_slots"] = reservedSlots.serializeNBT()
		nbt["matter_capability"] = matter.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		super.load(nbt)
		regularSlots.deserializeNBT(nbt["regular_slots"])
		reservedSlots.deserializeNBT(nbt["reserved_slots"])

		nbt.ifHas("matter_capability", CompoundTag::class.java) {
			matter.deserializeNBT(it)
		}
	}

	private var valid = true

	override fun invalidateCaps() {
		super.invalidateCaps()
		valid = false
		itemHandler.invalidate()
	}

	override fun reviveCaps() {
		super.reviveCaps()
		valid = true
		itemHandler.revive()
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (valid) {
			if (cap === MatteryCapability.MATTER_CELL) return resolverNode.cast()
			if (cap === CapabilityItemHandler.ITEM_HANDLER_CAPABILITY) return itemHandler.get().cast()
		}

		return super.getCapability(cap, side)
	}

	init {
		energy = MatteryMachineEnergyStorage(
			this,
			MatteryMachineEnergyStorage.MachineType.WORKER,
			Fraction(200000),
			Fraction(4000),
			Fraction(4000)
		)
	}

	companion object {
		private val NAME = TranslatableComponent("block.overdrive_that_matters.matter_replicator")
		private val BASE_CONSUMPTION = Fraction(400)
		private const val TICKS_PER_MTU = 20000.0
		private val TICKS_PER_MTU_BD = Fraction(20000)
		private const val MTU_PER_TICK = 1.0 / 20000.0
		private val MTU_PER_TICK_BD = Fraction.ONE.div(TICKS_PER_MTU_BD)
		private val DRAIN_MULT = Fraction(200)
	}
}