package ru.dbotthepony.mc.otm.client.screen.widget

import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.ChatFormatting
import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TranslatableComponent
import ru.dbotthepony.mc.otm.client.render.SkinElement
import ru.dbotthepony.mc.otm.client.screen.MatteryScreen
import ru.dbotthepony.mc.otm.client.screen.UVWindingOrder
import ru.dbotthepony.mc.otm.client.screen.panels.EditablePanel
import ru.dbotthepony.mc.otm.menu.FormattingHelper
import ru.dbotthepony.mc.otm.menu.widget.LevelGaugeWidget
import ru.dbotthepony.mc.otm.menu.widget.ProgressGaugeWidget

open class PowerGaugePanel @JvmOverloads constructor(
	screen: MatteryScreen<*>,
	parent: EditablePanel? = null,
	val widget: LevelGaugeWidget,
	x: Float = 0f,
	y: Float = 0f
): EditablePanel(screen, parent, x, y, width = GAUGE_BACKGROUND.rect_w, height = GAUGE_BACKGROUND.rect_h) {
	init {
		scissor = true
	}

	protected open fun makeTooltip(): MutableList<Component> {
		return mutableListOf(
			TranslatableComponent("otm.gui.power.percentage_level", String.format("%.2f", widget.percentage() * 100.0)),
			FormattingHelper.formatPowerLevel(widget.level(), widget.maxLevel())
		)
	}

	override fun innerRender(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float) {
		GAUGE_BACKGROUND.render(stack)
		val height = this.height * widget.percentage()
		GAUGE_FOREGROUND.renderPartial(stack, y = this.height - height, height = height, winding = UVWindingOrder.U0_V1_U1_V0)
	}

	override fun innerRenderTooltips(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float): Boolean {
		if (isHovered) {
			screen.renderComponentTooltip(stack, makeTooltip(), mouse_x.toInt(), mouse_y.toInt())
			return true
		}

		return false
	}

	companion object {
		val GAUGE_BACKGROUND = SkinElement(image_x = 0f, image_y = 48f, rect_w = 9f, rect_h = 48f)
		val GAUGE_FOREGROUND = SkinElement(image_x = 9f, image_y = 48f, rect_w = 9f, rect_h = 48f)
	}
}

open class MatterGaugePanel @JvmOverloads constructor(
	screen: MatteryScreen<*>,
	parent: EditablePanel? = null,
	val widget: LevelGaugeWidget,
	x: Float = 0f,
	y: Float = 0f
): EditablePanel(screen, parent, x, y, width = GAUGE_BACKGROUND.rect_w, height = GAUGE_BACKGROUND.rect_h) {
	init {
		scissor = true
	}

	protected open fun makeTooltip(): MutableList<Component> {
		return mutableListOf(
			TranslatableComponent("otm.gui.matter.percentage_level", String.format("%.2f", widget.percentage() * 100.0)),
			FormattingHelper.formatMatterLevel(widget.level(), widget.maxLevel())
		)
	}

	override fun innerRender(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float) {
		GAUGE_BACKGROUND.render(stack)
		val height = this.height * widget.percentage()
		GAUGE_FOREGROUND.renderPartial(stack, y = this.height - height, height = height, winding = UVWindingOrder.U0_V1_U1_V0)
	}

	override fun innerRenderTooltips(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float): Boolean {
		if (isHovered) {
			screen.renderComponentTooltip(stack, makeTooltip(), mouse_x.toInt(), mouse_y.toInt())
			return true
		}

		return false
	}

	companion object {
		val GAUGE_BACKGROUND = SkinElement(image_x = 0f, image_y = 0f, rect_w = 9f, rect_h = 48f)
		val GAUGE_FOREGROUND = SkinElement(image_x = 9f, image_y = 0f, rect_w = 9f, rect_h = 48f)
	}
}

open class PatternGaugePanel @JvmOverloads constructor(
	screen: MatteryScreen<*>,
	parent: EditablePanel? = null,
	val widget: LevelGaugeWidget,
	x: Float = 0f,
	y: Float = 0f
): EditablePanel(screen, parent, x, y, width = GAUGE_BACKGROUND.rect_w, height = GAUGE_BACKGROUND.rect_h) {
	init {
		scissor = true
	}

	protected open fun makeTooltip(): MutableList<Component> {
		return mutableListOf(
			TranslatableComponent("otm.gui.pattern.percentage_level", String.format("%.2f", widget.percentage() * 100.0)),
			TranslatableComponent("otm.gui.pattern.format", widget.level().decimalString(), widget.maxLevel().decimalString())
		)
	}

	override fun innerRender(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float) {
		GAUGE_BACKGROUND.render(stack)
		val height = this.height * widget.percentage()
		GAUGE_FOREGROUND.renderPartial(stack, y = this.height - height, height = height, winding = UVWindingOrder.U0_V1_U1_V0)
	}

	override fun innerRenderTooltips(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float): Boolean {
		if (isHovered) {
			screen.renderComponentTooltip(stack, makeTooltip(), mouse_x.toInt(), mouse_y.toInt())
			return true
		}

		return false
	}

	companion object {
		val GAUGE_BACKGROUND = SkinElement(image_x = 0f, image_y = 148f, rect_w = 9f, rect_h = 48f)
		val GAUGE_FOREGROUND = SkinElement(image_x = 9f, image_y = 148f, rect_w = 9f, rect_h = 48f)
	}
}

open class ProgressGaugePanel @JvmOverloads constructor(
	screen: MatteryScreen<*>,
	parent: EditablePanel? = null,
	val widget: ProgressGaugeWidget,
	x: Float = 0f,
	y: Float = 0f
): EditablePanel(screen, parent, x, y, width = GAUGE_BACKGROUND.rect_w, height = GAUGE_BACKGROUND.rect_h) {
	init {
		scissor = true
	}

	var flop = false

	protected open fun makeTooltip(): MutableList<Component> {
		val tooltip: MutableList<Component>

		if (widget.isStuck()) {
			tooltip = mutableListOf(
				TranslatableComponent(
					"otm.gui.progress_widget",
					String.format("%.2f", widget.percentage() * 100f)
				),
				TranslatableComponent("otm.gui.progress_widget_stuck").withStyle(ChatFormatting.DARK_RED)
			)
		} else {
			tooltip = mutableListOf(
				TranslatableComponent(
					"otm.gui.progress_widget",
					String.format("%.2f", widget.percentage() * 100f)
				)
			)
		}

		return tooltip
	}

	override fun innerRender(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float) {
		if (flop) {
			GAUGE_BACKGROUND.render(stack, winding = UVWindingOrder.U1_V0_U0_V1)
			val width = this.width * widget.percentage()
			GAUGE_FOREGROUND.renderPartial(stack, x = this.width - width, width = width, winding = UVWindingOrder.U1_V0_U0_V1)
		} else {
			GAUGE_BACKGROUND.render(stack)
			val width = this.width * widget.percentage()
			GAUGE_FOREGROUND.renderPartial(stack, width = width)
		}
	}

	override fun innerRenderTooltips(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float): Boolean {
		if (isHovered) {
			screen.renderComponentTooltip(stack, makeTooltip(), mouse_x.toInt(), mouse_y.toInt())
			return true
		}

		return false
	}

	companion object {
		val GAUGE_BACKGROUND = SkinElement(image_x = 0f, image_y = 132f, rect_w = 22f, rect_h = 16f)
		val GAUGE_FOREGROUND = SkinElement(image_x = 22f, image_y = 132f, rect_w = 22f, rect_h = 16f)
	}
}
